# -*- coding: utf-8 -*-
# Author      : ShiFan
# Created Date: 2024/9/18 09:44
"""The Module Has Been Build for..."""
import socket
import struct
import sys
import time

MCAST_GRP = '239.0.0.7'  # 多播组地址
MCAST_PORT = 5004         # 多播端口
BCAST_ADDR = ("255.255.255.255", 5005)  # 广播地址和端口
BCAST_PORT = 5005
MESSAGE = b"Hello, Broadcast!"


def multicast_send():
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
    sock.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, 2)

    while True:
        message = b"Hello, Multicast!"
        sock.sendto(message, (MCAST_GRP, MCAST_PORT))
        print(f"Message sent: {message}")
        time.sleep(1)


def multicast_recv():
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
    sock.bind(('', MCAST_PORT))

    mreq = struct.pack("4sl", socket.inet_aton(MCAST_GRP), socket.INADDR_ANY)
    sock.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, mreq)

    while True:
        data, addr = sock.recvfrom(1024)
        print(
            f"Received message: {data}<{type(data)}> from {addr}<{type(addr)}>"
        )


def broadcast_send():
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
    while True:
        sock.sendto(MESSAGE, BCAST_ADDR)
        print(f"Broadcast message sent: {MESSAGE}")
        time.sleep(1)


def broadcast_recv():
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
    sock.bind(("", BCAST_PORT))

    while True:
        data, addr = sock.recvfrom(1024)
        print(
            f"Received message: {data}<{type(data)}> from {addr}<{type(addr)}>"
        )


if __name__ == '__main__':
    if sys.argv[1] == "send":
        multicast_send()
    elif sys.argv[1] == "recv": 
        multicast_recv()
    elif sys.argv[1] == "bcast_send":
        broadcast_send()
    elif sys.argv[1] == "bcast_recv":
        broadcast_recv()



# -*- coding: utf-8 -*-
# Author      : ShiFan
# Created Date: 2024/9/25 17:29
import sys

import zmq
import time

def server():
    context = zmq.Context()

    # 创建一个 REP (reply) socket
    socket = context.socket(zmq.REP)
    socket.set(zmq.constants.SocketOption.IPV6, 1)

    # 使用 bind 方法和 IPv6 地址绑定到端口
    socket.bind("tcp://[::]:5555")  # 注意 IPv6 地址需要放在方括号内

    while True:
        # 接收消息
        message = socket.recv_string()
        print(f"Received request: {message}")

        # 发送回复
        socket.send_string("World")


def client():
    context = zmq.Context()

    # 创建一个 REQ (request) socket
    socket = context.socket(zmq.REQ)
    socket.set(zmq.constants.SocketOption.IPV6, 1)

    # 使用 connect 方法和 IPv6 地址连接到服务器
    socket.connect("tcp://[::]:5555")  # 注意 IPv6 地址需要放在方括号内

    for _ in range(10):
        # 发送请求
        print("Sending request ...")
        socket.send_string("Hello")

        # 接收回复
        message = socket.recv_string()
        print(f"Received reply: {message}")

        time.sleep(1)


if __name__ == "__main__":
    if sys.argv[1] == "client":
        client()
    elif sys.argv[1] == "server":
        server()


# -*- coding: utf-8 -*-
# Author      : ShiFan
# Created Date: 2024/8/14 10:57
import sys
from pathlib import Path

import zmq.constants as z_const


base_path = Path(__file__).parent
sys.path.append(base_path.parent.as_posix())

from gaterpc.utils import empty


DEBUG = 3
SECURE = True

ZMQ_SOCK = {
    z_const.SocketOption.IPV6: 1,
    z_const.SocketOption.HWM: 5000,
    # millisecond
    # z_const.SNDTIMEO: 10 * 1000,
    # millisecond
    # z_const.RCVTIMEO: 10 * 1000
}
# WORKER_ADDR: str = "tcp://0:777"

# GATE_IP_VERSION: int = 6
# GATE_MULTICAST_GROUP: str = "ff08::7"
# GATE_MULTICAST_PORT: int = 9777
# GATE_MULTICAST_HOP_LIMIT: int = 2

debug_format = (
    "%(asctime)s %(name)s "
    "[%(processName)s(%(process)d):%(threadName)s(%(thread)d)]\n"
    "%(pathname)s[%(funcName)s:%(lineno)d] \n"
    "- %(levelname)s %(message)s"
)

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "debug": {
            "()": "gaterpc.utils.ColorFormatter",
            "format": "%(asctime)s %(levelname)s %(name)s "
                      "[%(processName)s(%(process)d):"
                      "%(threadName)s(%(thread)d)]\n"
                      "%(pathname)s[%(funcName)s:%(lineno)d] "
                      "-\n%(message)s",
        },
        "verbose": {
            "()": "gaterpc.utils.ColorFormatter",
            "format": "%(asctime)s %(levelname)s %(name)s "
                      "%(module)s.[%(funcName)s:%(lineno)d] "
                      "-\n%(message)s",
        },
    },
    "handlers": {
        "test_gate": {
            "level": "DEBUG",
            "class": "gaterpc.utils.AQueueHandler",
            "handler_class": "logging.handlers.TimedRotatingFileHandler",
            "listener": "gaterpc.utils.singleton_aqueue_listener",
            "filename": empty,
            "formatter": "debug",
            "when": "midnight",
            "backupCount": 10
        },
        "console": {
            "level": "DEBUG",
            "class": "logging.StreamHandler",
            "formatter": "debug",
        },
    },
    "loggers": {
        "gaterpc": {
            "level": "DEBUG",
            "handlers": ["test_gate"],
            "propagate": True,
        },
        "gaterpc.zap": {
            "level": "DEBUG",
            "handlers": ["test_gate"],
            "propagate": False,
        },
        "gaterpc.worker": {
            "level": "DEBUG",
            "handlers": ["test_gate"],
            "propagate": False,
        },
        "gaterpc.client": {
            "level": "DEBUG",
            "handlers": ["console"],
            "propagate": False,
        },
        "commands": {
            "level": "DEBUG",
            "handlers": ["console"],
            "propagate": False,
        },
    },
}
